SystemOrganization addCategory: #'GT-Debugger'!

GLMBrowserTemplate subclass: #GTDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebugger commentStamp: 'TudorGirba 11/2/2012 22:07' prior: 0!
GTDebugger is a browser that provides Smalltalk debugging functionality.

An example:
[ GTDebugger debugProcess: Processor activeProcess.
	Processor activeProcess suspend.
	#(1 2 3 4 5) do: [:each | each asString]
] fork.!

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'AndreiChis 11/16/2012 10:38'!
debugContext: aContext fromProcess: aProcess

	<primitive: 19> "simulation guard"

	WorldState addDeferredUIMessage: [
		self openOn: (GTDebuggerSession process: aProcess context: aContext) ].! !

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'AndreiChis 11/25/2012 12:32'!
debugProcess: aProcess

	self debugContext: aProcess suspendedContext fromProcess: aProcess! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 10/31/2012 09:44'!
actOnBrowserClosing: ann
	self model terminate.! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 11/25/2012 12:36'!
buildBrowser
	browser := GLMTabulator new.
	browser title: 'Glamorous Debugger'.
	browser on: GLMBrowserClosing send: #actOnBrowserClosing: to: self.
	browser
		row: [ :row | 
					row
						column: #stack;
						column: #code ];
		row: #inspector.
	browser transmit
		to: #stack;
		andShow: [ :composite | self stackIn: composite ].
	browser transmit
		fromOutsidePort: #entity;
		to: #stack port: #selection;
		transformed: [:model | model context].
	browser transmit
		from: #stack port: #selection;
		fromOutsidePort: #entity;
		to: #stack port: #context;
		transformed: [ :context :model | model createModelForContext: context ].
	browser transmit
		from: #stack port: #selection;
		to: #stack port: #pc;
		transformed: #pc.
	browser transmit
		from: #stack port: #context;
		fromOutsidePort: #entity;
		to: #code;
		andShow: [ :composite | self methodCodeIn: composite ].
	browser transmit
		from: #stack;
		fromOutsidePort: #entity;
		from: #stack port: #pc;
		to: #code port: #selectionInterval;
		transformed: [ :context :model :pc | model selectedCodeRange: context ].
	browser transmit
		from: #stack port: #context;
		to: #inspector;
		andShow: [ :composite | self inspectorForContextAndSelfIn: composite ]! !

!GTDebugger methodsFor: 'building' stamp: 'TudorGirba 11/25/2012 22:55'!
inspectorForContextAndSelfIn: composite
	composite dynamic
		display: [ :each | 
			GTInspector new
				noTitle;
				noActions;
				showFirst: [ :a | a custom: (GTDebuggerVariablesBrowser new browser startOn: each) ];
				startOn: each ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installDebuggingActionsFor: aPresentation
	self installRestartActionFor: aPresentation.
	self installStepIntoActionFor: aPresentation.
	self installStepOverActionFor: aPresentation.
	self installStepThroughActionFor: aPresentation.
	self installResumeActionFor: aPresentation ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:14'!
installRestartActionFor: aPresentation 
	aPresentation 
		act: [ :presentation :model |
			model restart: self selectedContext.
			self selectTopOfStack: model context  ]
		icon: GLMUIThemeExtraIcons glamorousRestart
		entitled: 'Restart'! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 11/20/2012 12:14'!
installResumeActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model resume.
			model clear.
			browser close. ]
		icon: GLMUIThemeExtraIcons glamorousPlay
		entitled: 'Resume' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepIntoActionFor: aPresentation 
	aPresentation 
		act: [ :presentation :model |
			model stepInto: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousInto
		entitled: 'Step into' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepOverActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model stepOver: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousOver
		entitled: 'Step over' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepThroughActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model stepThrough: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousThrough
		entitled: 'Step through' ! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 10/23/2012 15:56'!
keyAsFormatedString: key

	^ [ Text fromString: (key asString contractTo: 50) ]
			on: Error
			do: [ Text string: 'error printing' attribute: TextColor red ] ! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 11/4/2012 21:13'!
methodCodeIn: composite
	composite smalltalkCode
		act: [:text :contextModel | 
				| recompilationContext newMethod |				
				(recompilationContext := contextModel locateClosureHome) isNil ifFalse: [
					newMethod := self model recompileMethodTo: text text inContext: recompilationContext.
					newMethod isNil ifFalse: [ self selectTopOfStack: self model context ] ] ]
			icon: GLMUIThemeExtraIcons glamorousAccept 
			on: $s
			entitled: 'Accept';
		doItReceiver: [:contextModel | contextModel receiver ];
		title: 'Source';
		format: #sourceText;
		smalltalkClass: [ :contextModel | contextModel selectedClass ]! !

!GTDebugger methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 16:00'!
model
	^ (browser pane port: #entity) value! !

!GTDebugger methodsFor: 'private' stamp: 'TudorGirba 11/2/2012 22:26'!
selectTopOfStack: aContext
	| stackPane |
	stackPane := browser paneNamed: #stack.
	(stackPane port: #selection) value: aContext.
	(stackPane port: #pc) value: aContext pc.
	stackPane update! !

!GTDebugger methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 18:28'!
selectedContext
	^ ((browser paneNamed: #stack) port: #selection) value! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 11/25/2012 12:09'!
stackIn: composite
	composite list	
		title: 'Stack';
		showOnly: 10;
		display: #stack;
		format: [ :each | String streamContents: [:stream | each printDebugOn: stream ] ];
		with: [ :list | self installDebuggingActionsFor: list ]! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 10/23/2012 15:58'!
valueAsFormatedString: value

	^ [ Text fromString: value printString]
			on: Error
			do: [ Text string: 'error printing' attribute: TextColor red ] ! !

GLMBrowserTemplate subclass: #GTDebuggerVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebuggerVariablesBrowser commentStamp: 'TudorGirba 11/2/2012 22:06' prior: 0!
GTDebuggerVariablesBrowser is a special browser that is used in the first pane of the inspector that is embedded in the debugger. Its main intention is to provide a unified list of variables that come from:
- the instance variables of the receiver
- the local variables from the context!

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'TudorGirba 11/2/2012 21:37'!
buildBrowser 
	browser := GLMTabulator new.
	browser title: [:each | GTInspector new printObjectAsAnItem: each receiver ].
	browser 
		morphicAct: [:anotherComposite | anotherComposite entity receiver browse]
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			entitled: 'Browse'.
	browser row: #variables; row: #evaluator.
	browser transmit to: #variables; andShow: [:a | 
		self variablesIn: a ].
	browser transmit to: #evaluator; andShow: [:a | 
		self evaluatorIn: a ].
	browser transmit from: #variables; toOutsidePort: #selection.
	browser transmit from: #evaluator; toOutsidePort: #selection.
	^ browser ! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 10/30/2012 21:03'!
evaluatorIn: composite
	(composite smalltalkCode)
		doItReceiver: #yourself;
		display: [ :model | [model receiver printString asSmalltalkComment, String cr, 'self'] on: Error do: ['"Error printing. Try self printString to debug"'] ];
		selectionPopulate: #selection
			on: $o
			entitled: 'Do it and open (o)'
			with: [ :text :model | Compiler evaluate: text selectedText for: model receiver logged: false ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 11/2/2012 22:10'!
variablesIn: composite
	composite table
		showOnly: 50;
		display: [ :contextModel | contextModel gtDebuggerRetrieveVariableValuePairs ];
		sorted: [:variablePair1 :variablePair2 | variablePair1 key < variablePair2 key];
		column: 'Variable' evaluated: [:assoc | 
			[ Text fromString: assoc key asString ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
		column: 'Value' evaluated: [:assoc | 
			[ Text fromString: (assoc value printString contractTo: 50) ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
"		icon: [ :each |
			each value class theNonMetaClass browserIcon
				ifNotNil: [ OBMorphicIcons iconNamed: each value class theNonMetaClass browserIcon ] ];"
		send: #value;
		morphicSelectionAct: [:each | each value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse'! !

!PreDebugWindow methodsFor: '*gt-debugger' stamp: 'TudorGirba 11/1/2012 21:23'!
spawnGTDebugger
	model spawnGTDebugger! !

Object subclass: #GTDebugContext
	instanceVariableNames: 'context topContext method methodNode ranges'
	classVariableNames: 'MapCache MapCacheEntries'
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebugContext commentStamp: 'TudorGirba 11/2/2012 22:08' prior: 0!
GTDebugContextModel is a helper that complements the GTDebuggerModel. It is meant to be created dynamically for each selected method.

Instance Variables:
	context	<ContextPart>
	topContext	<ProtoObject>
	method	<CompiledMethod>
	methodNode	<MethodNode>
	ranges	<Dictionary>!

!GTDebugContext classSide methodsFor: 'instance creation' stamp: 'AndreiChis 11/1/2012 13:32'!
forContext: aContext 
	^ self new forContext: aContext ! !

!GTDebugContext methodsFor: 'evaluating actions' stamp: 'AndreiChis 10/30/2012 19:21'!
evaluate: expression
	^ Compiler new
		evaluate: expression
		in: context
		to: context receiver! !

!GTDebugContext methodsFor: 'initialization' stamp: 'AndreiChis 11/1/2012 13:39'!
forContext: aContext 
	method := aContext method.
	methodNode := method methodNode.

	ranges := Dictionary new.
	
	" Put all ranges with a possitive Key PC in the ranges dictionary "
	methodNode rawSourceRanges
		keysAndValuesDo: [ :node :range |
			node pc = 0
				ifFalse: [ ranges at: node pc put: range ]].
			
	context := aContext.
	topContext := aContext.

	" Sort all the Key PCs in the sortedKeyPCs "
	"sortedKeyPCs := SortedCollection new."
	" ranges keysDo: [ :pc | sortedKeyPCs add: pc ]."! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/7/2012 11:59'!
gtDebuggerRetrieveVariableValuePairs
	|fieldList|

	fieldList := OrderedCollection new.
	fieldList 
		add: ('_self' ->  context receiver);
	      addAll: (context receiver class allInstVarNames collect: [:each | each -> (context receiver instVarNamed: each) ]).
	
	fieldList add: ('_thisContext' ->  context).
	 context == nil ifTrue: [^fieldList].
	fieldList 
		add: ('_stack top' -> ( context stackPtr > 0 ifTrue: [ context top])).
		
	(context tempNames size > 0) ifTrue: [
		fieldList addAll: ((1 to: context tempNames size) collect: [ :index || key value |
			key := context tempNames at: index.
			value := [context debuggerMap namedTempAt: index in: context] 
				on: Error 
				do: [value := (Text string: 'error obtaining field value' attribute: TextColor red) asString ].
			key -> value ] ) ].
	
	^ fieldList! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/4/2012 21:12'!
locateClosureHome
	"In case the current context is a BlockContext locate the closureHome and ask the
	user to validate the new context. If closureHome is not found or the user does not 
	validate the new context, return nil. 
	If the current context is not a BlockContext return it."
	| closureHome result aText |
	
	context isExecutingBlock ifTrue: [
		closureHome := context activeHome.
		closureHome ifNil: [
			self inform: 'Method for block not found on stack, can''t edit and continue'.
		 	^ nil].
		 (self confirm: 'I will have to revert to the method from\which this block originated.  Is that OK?' withCRs) 
			ifFalse: [^ nil].
		^ closureHome].
	
	^ context! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2012 21:02'!
receiver
	^ context receiver! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/2/2012 12:18'!
receiverClass
	"Answer the class of the receiver. 
	It may differ from 'self selectedClass' "
	
	^ context receiver class! !

!GTDebugContext methodsFor: 'evaluating actions' stamp: 'AndreiChis 11/4/2012 21:32'!
recompileCurrentMethodTo: aText 
	| classOfMethod selector |
	
	classOfMethod := self selectedClass.
	selector := classOfMethod parserClass new parseSelector: aText.
	(selector == self selectedMessageName or: [(self selectedMessageName beginsWith: 'DoIt')
		and: [selector numArgs = self selectedMessageName numArgs]]) ifFalse: [
			self inform: 'can''t change selector'. ^ nil].
		
	selector := classOfMethod 
		compile: aText 
		classified: self selectedMessageCategoryName 
		notifying: nil.
	selector isNil 
		ifFalse: [ ^ classOfMethod compiledMethodAt: selector. ]
		ifTrue: [ ^ nil ] 
		! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/4/2012 21:23'!
selectedClass
	"Answer the class in which the current context's method was found."

	^ context methodClass! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/4/2012 21:24'!
selectedMessageCategoryName
	"Answer the name of the message category of the message of the current context."

	^ self selectedClass organization categoryOfElement: self selectedMessageName! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/4/2012 21:32'!
selectedMessageName
	"Answer the message selector of the current context.
	 If the method is unbound we can still usefully answer its old selector."

	| selector |
	selector := context methodSelector.
	^(selector ~~ context method selector
	    and: [selector beginsWith: 'DoIt'])
		ifTrue: [context method selector]
		ifFalse: [selector]! !

!GTDebugContext methodsFor: 'private' stamp: 'AndreiChis 10/30/2012 16:47'!
source
	^ method holdsTempNames
		ifTrue: [ method getSource ]
		ifFalse: [ methodNode sourceText ]! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2012 16:47'!
sourceRangeForPC: pc
	^ ranges at: pc ifAbsent: [ ^ 1 to: 0 ].! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2012 16:47'!
sourceText
	^ self source asText! !

!GTDebugContext methodsFor: 'accessing' stamp: 'AndreiChis 11/1/2012 13:40'!
topContext: aContext
	topContext  := aContext ! !

Object subclass: #GTDebuggerSession
	instanceVariableNames: 'context process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebuggerSession commentStamp: 'TudorGirba 11/2/2012 22:01' prior: 0!
This models the debugger. It contains the current context and process. Its main goal is to handle debugger actions such as restart or stepInto. It is the model used as an input to GTDebugger.

Instance Variables:
	context	<ContextPart>
	process	<Process>!

!GTDebuggerSession classSide methodsFor: 'instance creation' stamp: 'AndreiChis 11/1/2012 11:54'!
process: aProcess context: aContext
	^ self new
		process: aProcess context: aContext! !

!GTDebuggerSession methodsFor: 'private process' stamp: 'ToonVerwaest 3/12/2011 16:54'!
activePC: aContext
	^ (self isLatestContext: aContext)
		ifTrue: [ context pc ]
		ifFalse: [ self previousPC: aContext ].! !

!GTDebuggerSession methodsFor: 'private' stamp: 'AndreiChis 11/1/2012 14:24'!
clear
	process := nil.
	context := nil! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 10:13'!
context
	^ context! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'AndreiChis 11/25/2012 12:36'!
createModelForContext: aContext
	^ (GTDebugContext forContext: aContext) topContext: context! !

!GTDebuggerSession methodsFor: 'private context' stamp: 'AndreiChis 11/25/2012 12:36'!
currentCodeRange: aContext
	^(self createModelForContext: aContext) sourceRangeForPC: (self activePC: aContext)! !

!GTDebuggerSession methodsFor: 'private context' stamp: 'AndreiChis 11/25/2012 12:12'!
filterTopContext: aContext
	|ctx|
	 
	ctx := aContext.
	(self shouldDisplayOnTopContext: ctx) ifTrue: [^ ctx].
	[(ctx := ctx sender) ~~ nil] whileTrue:
		[(self shouldDisplayOnTopContext: ctx) ifTrue: [^ ctx]].
	^ nil! !

!GTDebuggerSession methodsFor: 'private process' stamp: 'ToonVerwaest 3/12/2011 16:52'!
isLatestContext: aContext
	^ process suspendedContext == aContext! !

!GTDebuggerSession methodsFor: 'private context' stamp: 'ToonVerwaest 3/12/2011 16:50'!
previousPC: aContext
	^ (aContext method pcPreviousTo: aContext pc)
		ifNil: [ aContext pc ]! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 10:13'!
process
	^ process! !

!GTDebuggerSession methodsFor: 'initialization' stamp: 'AndreiChis 11/25/2012 12:27'!
process: aProcess context: aContext
	aProcess stepToSendOrReturn.
	process := aProcess.
	context := self filterTopContext: aContext.
	context isNil ifTrue: [context := aContext].
	
! !

!GTDebuggerSession methodsFor: 'evaluating actions' stamp: 'AndreiChis 11/25/2012 12:36'!
recompileMethodTo: text inContext: recompilationContext
	| newMethod |				
				
	newMethod := (self createModelForContext: recompilationContext) recompileCurrentMethodTo: text.
	newMethod isNil 
		ifFalse: [ 
			self rewindContextToMethod: newMethod fromContext: recompilationContext. 
			^ newMethod ] 
		ifTrue: [
			^ nil ]! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'ToonVerwaest 3/12/2011 12:45'!
restart: selectedContext
	self unwindTo: selectedContext.
	process
		restartTop;
		stepToSendOrReturn! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 11/25/2012 12:28'!
resume
	process resume.! !

!GTDebuggerSession methodsFor: 'updating' stamp: 'AndreiChis 11/4/2012 21:13'!
rewindContextToMethod: aMethod fromContext: aContext
	"this method is typically to be used after a hot compilation of a method from the stack"
	| ctxt |
	ctxt :=  process popTo: aContext.
	
	ctxt == aContext
		ifFalse: [
			self inform: 'Method saved, but current context unchanged\because of unwind error. Click OK to see error' withCRs]
		ifTrue: [
			process
				restartTopWith: aMethod;
				stepToSendOrReturn ].
	
	self flag: 'is it realy necessary to handle quick methods separately?'.
	
	^ context := ctxt! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 18:40'!
selectedCodeRange: selectedContext
	^ self currentCodeRange: selectedContext! !

!GTDebuggerSession methodsFor: 'private context' stamp: 'AndreiChis 11/25/2012 12:11'!
shouldDisplayOnTopContext: aContext
	
	^ aContext method selector ~= #halt! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'AndreiChis 11/20/2012 17:14'!
stack
	^ context stack! !

!GTDebuggerSession methodsFor: 'accessing' stamp: 'ToonVerwaest 3/12/2011 11:45'!
stackOfSize: size
	^ context stackOfSize: size! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'ToonVerwaest 3/12/2011 13:28'!
stepInto
	self stepInto: context! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'ToonVerwaest 3/12/2011 13:33'!
stepInto: selectedContext
	context := process step: selectedContext.
	process stepToSendOrReturn.! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'ToonVerwaest 3/12/2011 13:29'!
stepOver
	self stepOver: context! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 11/1/2012 15:05'!
stepOver: selectedContext
	| newContext |
	newContext := process completeStep: selectedContext.
	context := 
		newContext == selectedContext
			ifTrue: [ process stepToSendOrReturn ]
			ifFalse: [ newContext ]! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'ToonVerwaest 3/16/2011 11:19'!
stepThrough
	self stepThrough: context! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 11/1/2012 14:49'!
stepThrough: selectedContext
	"Send messages until you return to the present method context.
	 Used to step into a block in the method."
	
	context := process stepToHome: selectedContext.
	process stepToSendOrReturn.! !

!GTDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 11/1/2012 14:01'!
terminate
	self process isNil ifFalse: [self process terminate].
	self clear.! !

!GTDebuggerSession methodsFor: 'private' stamp: 'ToonVerwaest 3/12/2011 12:45'!
unwindTo: aContext
	|ctx|
	ctx := process popTo: aContext.
	ctx == aContext ifFalse: [ self error: 'Could not unwind stack' ].
	context := aContext! !

!GTDebuggerSession methodsFor: 'updating' stamp: 'AndreiChis 11/20/2012 18:55'!
updateWithContext: newContext fromProcess: aProcess
	self process: aProcess context: newContext
	 
	"self process: aProcess.
	self unwindTo: (self filterTopContextAndUnwindForContext: newContext).
	self process stepToSendOrReturn."! !

Debugger subclass: #GTProxyDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTProxyDebugger commentStamp: 'TudorGirba 11/2/2012 22:04' prior: 0!
GTProxyDebugger is a simple extension of the existing debugger that adds a button for "debug glamorously" using the GTDebugger in the regular debugger.

The goal is to provide the possibility of having the GTDebugger next to the classic debugger until the inevitable errors are ironed out.!

!GTProxyDebugger classSide methodsFor: 'class initialization' stamp: 'AndreiChis 11/1/2012 15:13'!
initialize
	super initialize.

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools register: self as: #debugger! !

!GTProxyDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/25/2012 12:52'!
debuggerClass
	^ GTDebugger ! !

!GTProxyDebugger methodsFor: 'initialize' stamp: 'TudorGirba 11/1/2012 21:23'!
preDebugButtonQuads
	| buttons |
	self isPostMortem 
		ifTrue: [ ^ self postMortemPreDebugButtonQuads ].
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'    		translated.		#debug.		#red. 	'Bring up a debugger' translated}.
		{'Debug glamorously' translated. #spawnGTDebugger. #red. 'Bring up the Glamorous Debugger' translated}
	}! !

!GTProxyDebugger methodsFor: 'notifier menu' stamp: 'AndreiChis 11/20/2012 12:11'!
spawnGTDebugger
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	
	self debuggerClass debugContext: self interruptedContext fromProcess: self interruptedProcess! !
GTProxyDebugger initialize!
